[
  {
    text: 'Conditionals',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'If - Then',
        leaf: true,
        script: "if (condition) then\n  dosomething()\nend"
      },
      {
        text: 'If (AND) - Then',
        leaf: true,
        script: "if (condition1 and condition2) then\n  dosomething()\nend"
      },
      {
        text: 'If (OR) - Then',
        leaf: true,
        script: "if (condition or condition2) then\n  dosomething()\nend"
      },
      {
        text: 'If - Then - Else',
        leaf: true,
        script: "if (condition) then\n  dosomething()\nelse\n  dosomethingelse()\nend"
      },
      {
        text: 'If - Else If',
        leaf: true,
        script: "if (condition) then\n  dosomething()\nelseif (othercondition) then\n  dosomethingelse()\nend"
      }
    ]
  },
  {
    text: 'Loops and iterators',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Hashtable iteration',
        leaf: true,
        script: "-- hashtable = { a = 1, b = 2, c = 3 }\nfor key, value in pairs(hashtable) do\n  dosomething()\nend"
      },
      {
        text: 'Array iteration',
        leaf: true,
        script: "-- arraytable = { 'a', 'b', 'c' }\nfor index, value in ipairs(arraytable) do\n  dosomething()\nend"
      },
      {
        text: 'Numeric for loop',
        leaf: true,
        script: "-- for i = START, STOP, STEP\nfor i = 1, 10, 2 do\n  dosomething()\nend"
      },
      {
        text: 'While loop',
        leaf: true,
        script: "-- loop while condition is met\nwhile (condition) do\n  dosomething()\nend"
      },
      {
        text: 'Repeat..until loop',
        leaf: true,
        script: "-- loop until condition is met\nrepeat\n  dosomething()\nuntil (condition)"
      }
    ]
  },
  {
    text: 'Math',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Absolute value',
        leaf: true,
        script: "-- e.g. x = -12, y = 12\ny = math.abs(x)"
      },
      {
        text: 'Ceiling',
        leaf: true,
        script: "-- e.g. x = 12.2, y = 13\ny = math.ceil(x)"
      },
      {
        text: 'Floor',
        leaf: true,
        script: "-- e.g. x = 12.7, y = 12\ny = math.floor(x)"
      },
      {
        text: 'Round to integer',
        leaf: true,
        script: "-- e.g. x = 12.7, y = 13; x = 12.3, y = 12\ny = math.floor(x + 0.5)"
      },
      {
        text: 'Random value',
        leaf: true,
        script: "-- get random integer value between min and max (inclusive)\ny = math.random(min, max)"
      }
    ]
  },
  {
    text: 'Objects / KNX bus',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Get event value',
        leaf: true,
        script: "-- object mapped to this event must have its data type set\nvalue = event.getvalue()"
      },
      {
        text: 'Get object value (by name)',
        leaf: true,
        script: "-- get value of object named 'my object'\nvalue = grp.getvalue(|>'my object'<|)"
      },
      {
        text: 'Get object value (by address)',
        leaf: true,
        script: "-- get value of object with group address 1/1/1\nvalue = grp.getvalue(|>'1/1/1'<|)"
      },
      {
        text: 'Group read (by name)',
        leaf: true,
        script: "-- send read request to object named 'my object'\ngrp.read(|>'my object'<|)"
      },
      {
        text: 'Group read (by address)',
        leaf: true,
        script: "-- send read request to 1/1/1\ngrp.read(|>'1/1/1'<|)"
      },
      {
        text: 'Group write (by name, constant)',
        leaf: true,
        script: "-- write value (e.g. boolean 'true') to object named 'my object', datatype must be set for this object\ngrp.write(|>'my object'<|, true)"
      },
      {
        text: 'Group write (by address, constant)',
        leaf: true,
        script: "-- write value (e.g. boolean 'true') to object with group address 1/1/1, datatype must be set for this object\ngrp.write(|>'1/1/1'<|, true)"
      },
      {
        text: 'Group write (by name, variable)',
        leaf: true,
        script: "-- write value (e.g. number '12.1') to object named 'my object', datatype must be set for this object\nvalue = 12.1\ngrp.write(|>'my object'<|, value)"
      },
      {
        text: 'Group write (by address, variable)',
        leaf: true,
        script: "-- write value (e.g. number '12.1') to object with group address 1/1/1, datatype must be set for this object\nvalue = 12.1\ngrp.write(|>'1/1/1'<|, value)"
      },
      {
        text: 'Update internal value (by name)',
        leaf: true,
        script: "-- no KNX/TP telegram is sent, only database value is changed\ngrp.update(|>'my object'<|, 123)"
      },
      {
        text: 'Update internal value (by address)',
        leaf: true,
        script: "-- no KNX/TP telegram is sent, only database value is changed\ngrp.update(|>'1/1/1'<|, 123)"
      },
      {
        text: 'Convert name to group address',
        leaf: true,
        script: "address = grp.alias(|>'my object'<|)"
      },
      {
        text: 'Convert group address to name',
        leaf: true,
        script: "name = grp.alias(|>'1/1/1'<|)"
      },
      {
        text: 'Single object functions',
        cls: 'folder',
        expanded: false,
        children: [
          {
            text: 'Find object (by address)',
            leaf: true,
            script: "-- get object with address 1/1/1\nmyobject = grp.find(|>'1/1/1'<|)"
          },
          {
            text: 'Find object (by name)',
            leaf: true,
            script: "-- get object with name 'my object'\nmyobject = grp.find(|>'my object'<|)"
          },
          {
            text: 'Write value to bus',
            leaf: true,
            script: "-- write value (e.g. 127) to the object\nmyobject:write(127)"
          },
          {
            text: 'Send read request to bus',
            leaf: true,
            script: "-- write value (e.g. 127) to the object\nmyobject:read()"
          }
        ]
      },
      {
        text: 'Multiple object functions (tags)',
        cls: 'folder',
        expanded: false,
        children: [
          {
            text: 'Find by single tag',
            leaf: true,
            script: "-- get an array of objects tagged 'tag name'\nmyobjects = grp.tag(|>'tag name'<|)"
          },
          {
            text: 'Find by multiple tags (match all)',
            leaf: true,
            script: "-- get an array of objects tagged 'tag name 1' AND 'tag name 2'\nmyobjects = grp.tag({'tag name 1', 'tag name 2'}, 'all')"
          },
          {
            text: 'Find by multiple tags (match any)',
            leaf: true,
            script: "-- get an array of objects tagged 'tag name 1' OR 'tag name 2'\nmyobjects = grp.tag({'tag name 1', 'tag name 2'}, 'any')"
          },
          {
            text: 'Write value to bus',
            leaf: true,
            script: "-- write value (e.g. 127) to the object\nmyobjects:write(127)"
          },
          {
            text: 'Send read request to bus',
            leaf: true,
            script: "-- read all tagged objects\nmyobjects:read()"
          }
        ]
      }
    ]
  },
  {
    text: 'Storage',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Get data from storage',
        leaf: true,
        script: "-- mydata will be set nil when data was not found\nmydata = storage.get(|>'myobjectdata'<|)"
      },
      {
        text: 'Get data (with default value)',
        leaf: true,
        script: "-- mydata will be set to a predefined value (e.g. 127) when data was not found\nmydata = storage.get(|>'myobjectdata'<|, 127)"
      },
      {
        text: 'Save data to storage',
        leaf: true,
        script: "-- set storage variable myobjectdata to a specified value (e.g. 127)\nstorage.set(|>'myobjectdata'<|, 127)"
      }
    ]
  },
  {
    text: 'Script control',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Enable script',
        leaf: true,
        script: "script.enable(|>'scriptname'<|)"
      },
      {
        text: 'Disable script',
        leaf: true,
        script: "script.disable(|>'scriptname'<|)"
      },
      {
        text: 'Get script status',
        leaf: true,
        script: "-- return true/false if script is found, nil otherwise\nstatus = script.status(|>'scriptname'<|)"
      }
    ]
  },
  {
    text: 'Alerts and logs',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Alert',
        leaf: true,
        script: "alert('|>My alert text<|')"
      },
      {
        text: 'Formatted alert',
        leaf: true,
        script: "-- see string.format for more info\nalert('Group address %s, numeric value: %d', '1/1/1', 127)"
      },
      {
        text: 'Log variables',
        leaf: true,
        script: "a = { f1 = 10, f2 = 20 }\nb = 127\nc = 'test'\n-- all passed arguments will appear in Logs module\nlog(a, b, c)"
      }
    ]
  },
  {
    text: 'Time functions',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Delay script execution',
        leaf: true,
        script: "-- wait for 1.5 seconds\nos.sleep(|>1.5<|)"
      }
    ]
  },
  {
    text: 'Miscellaneous',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Send an e-mail',
        leaf: true,
        script: "-- make sure mail settings are set in user function library before using this function\nsubject = 'E-mail test'\nmessage = 'Testing e-mail'\nmail('user@example.com', subject, message)"
      },
      {
        text: 'Sunrise / sunset',
        leaf: true,
        script: "-- returns number of minutes from 0:00 until sunrise and sunset\nlatitude = 57\nlongitude = 24.15\nsunrise, sunset = rscalc(latitude, longitude)"
      },
      {
        text: 'Convert sunrise / sunset to readable time',
        leaf: true,
        script: "sunrise = string.format('%d:%.2d', math.floor(sunrise / 60), sunrise % 60)\nsunset = string.format('%d:%.2d', math.floor(sunset / 60), sunset % 60)"
      }
    ]
  },
  {
    text: 'Serial',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Open connection',
        leaf: true,
        script: "require('serial')\nport = serial.open('/dev/RS232', {\n  baudrate = 38400,\n  databits = 8,\n  stopbits = 1,\n  parity = 'none',\n  duplex = 'full'\n})"
      },
      {
        text: 'Write to port',
        leaf: true,
        script: "port:write('test data')"
      },
      {
        text: 'Blocking read',
        leaf: true,
        script: "-- script will block until 10 characters are read\ndata = port:read(10)"
      },
      {
        text: 'Timeout read',
        leaf: true,
        script: "-- script will wait for 10 characters for 20 seconds\ndata = port:read(10, 20)"
      },
      {
        text: 'Close serial port',
        leaf: true,
        script: "port:close()"
      }
    ]
  },
  {
    text: 'Modbus',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Create Modbus TCP object',
        leaf: true,
        script: "require('luamodbus')\nmb = luamodbus.tcp()"
      },
      {
        text: 'Create Modbus RTU object',
        leaf: true,
        script: "require('luamodbus')\nmb = luamodbus.rtu()"
      },
      {
        text: 'Open Modbus TCP connection',
        leaf: true,
        script: "-- IP: 192.168.1.2, port: 1234\nmb:open('192.168.1.2', 1234)\nmb:connect()"
      },
      {
        text: 'Open Modbus RTU connection',
        leaf: true,
        script: "-- 19200 baud rate, even parity, 8 data bits, 1 stop bit, half duplex\nmb:open('/dev/RS485', 19200, 'E', 8, 1, 'H')\nmb:connect()"
      },
      {
        text: 'Set slave address',
        leaf: true,
        script: "mb:setslave(123)"
      },
      {
        text: 'Read single coil',
        leaf: true,
        script: "-- read 1 coil from address 1000\ncoil = mb:readcoils(1000)"
      },
      {
        text: 'Read multiple coils',
        leaf: true,
        script: "-- read 3 coils from address 1000\ncoil1, coil2, coil3 = mb:readcoils(1000, 3)"
      },
      {
        text: 'Read discrete input',
        leaf: true,
        script: "-- read from address 1000\nvalue = mb:readdiscreteinputs(1000)"
      },
      {
        text: 'Read registers',
        leaf: true,
        script: "-- read from address 1000\nvalue = mb:readregisters(1000)"
      },
      {
        text: 'Read input registers',
        leaf: true,
        script: "-- read from address 1000\nvalue = mb:readinputregisters(1000)"
      },
      {
        text: 'Write single bit',
        leaf: true,
        script: "-- set bit at address 1000 to 'on'\nmb:writebits(1000, true)"
      },
      {
        text: 'Write multiple bits',
        leaf: true,
        script: "-- set bit at address 1000 to 'on', bit at address 1001 to 'off'\nmb:writebits(1000, true, false)"
      },
      {
        text: 'Write single register',
        leaf: true,
        script: "-- set register at address 1000 to 123\nmb:writeregisters(1000, 123)"
      },
      {
        text: 'Write multiple registers',
        leaf: true,
        script: "-- set register at address 1000 to 123, register at address 1001 to 321\nmb:writeregisters(1000, 123, 321)"
      },
      {
        text: 'Close modbus connection',
        leaf: true,
        script: "mb:close()"
      }
    ]
  },
  {
    text: 'KNXlogic 1.3',
    cls: 'folder',
    expanded: false,
    children: [
	  {
		text: 'sms functions',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'sms daemon basic (required)',
			leaf: true,
			cls: 'file',
			script: "--for all sms functionality it is required to run the deamon in a resident script with delay 1\n--the deamon assures that sms are send and received/processed by the default rules\n\nrequire 'sms'\n\nif not first then\n  usbport = '/dev/ttyUSB2'\n  first=true\nend\nsms.run(usbport,'specify your SIM pin here if applicable')\n\n--fallback, in case configured USB port changed\nif not sms.running then \n  modemsfound = sms.findModems('USB')\n  log(modemsfound) \n  for k,t in pairs(modemsfound) do\n  	usbport = t.port\n    alert('SMS: Found modem on port '..usbport..'. Trying to connect.')\n    break\n  end\nend\n"
		  },
		  {
			text: 'find the sms modem port',
			leaf: true,
			cls: 'file',
			script: "--Logs all found modems so you can find the port of the modem you want to use for sms sending\n--Add it temporarily to the deamon script\n--In the modem info you will also see if the simcard is pin-locked or not\n--Give the call 'USB' or 'serial' as argument if you only want to search USB/serial modems. No argument = serial and USB\nlog(sms.findModems('USB'))"
		  },
		  {
			text: 'set ack message options',
			leaf: true,
			cls: 'file',
			script: "--to be added to the main sms daemon script\nsms.acktext = 'ACK' --default value, text is not case sensitive\nsms.ackinfo = '\\nReply ACK to confirm' --default value, sets the message which is added to a sms requiring an ack, to inform the recipient"
		  },
		  {
			text: 'daemon for sms sending on many KNX objects or telegrams',
			leaf: true,
			cls: 'file',
			script: "-- the following resident script (interval 5-60s is OK) will send sms on all wanted events of objects.\n-- this avoids to have many separate script to manage this\n-- make sure you have the generic SMS daemon running, because the actual sending of the SMS is done by that daemon\n-- run this as resident script with 1-10s interval\n\nif not running then\n  require('sms')\n\n  ----------------------------------\n  --START of user configuration area\n  --for each object event you want to define an sms, add a table/line below\n  --each sms event if characterised by:\n  --  ga = the group address you want to use as trigger (a string like '2/1/4')\n  --  tel = a table of all telephone numbers that must receive the sms. If acktime is used, then the table must have numbered telnrs n1, ...\n  --  msg = the string that must be sent with the sms. Inside the text you can use\n  --     $ev : will be replaced by the object event value or telegram value\n  --     $d : will be replaced by the date\n  --     $t : will be replaced by the time\n  --     $dt : will be replaced by the date and time\n  --  flash : can be true of false (default false). The will create a 'flash sms' instead of a regular one\n  --  acktime : can be a positive number (default: no ack). This results in an ack sms reply to be expected and escalation if no ack received in time\n  --  active: tell if the sms must be send. Possible values:\n  --    boolean: true or false : the message will be always/never sent\n  --    table: the values in the table will be compared with the event value (or telegram value). String based comparison.\n  --           Only when the event value is in the table, the sms is sent.\n  --    default value: true\n  --  msgfu : for more advanced messages you can specify a function with takes 'event' as the argument and returns a msg string\n  --          if both msg and msgfu are given, both string are concatenated (first msg, then msgfu). Default = nil\n  --  activefu : option not yet availble\n  --\n  -- example line: see commented line below\n  --     {ga='11/5/0', tel={'+32123123123'},msg='BBD: Burglary alarm in zone $ev on $t',flash=true},\n  --     {ga='11/5/0', tel={n1='+32123123123',n2='+32123123124',n3='+32123123125'},msg='BBD: Burglary alarm in zone $ev on $t',flash=false, acktime=120},\n\n  \n  smscfgmap = {\n  --     {ga='11/5/0', tel={'+32123123123'},msg='BBD: Burglary alarm in zone $ev on $t',flash=true},\n  }\n  \n  \n  --END of user configuration area \n  --do not modify anything below!\n  --------------------------------------\n  \n  sms.KNXsender.init(smscfgmap)\n\n  running=true\nend\n\nsms.KNXsender.run()\n"
		  },
		  {
			text: 'send a simple sms',
			leaf: true,
			cls: 'file',
			script: "--assumes the daemon is running\nrequire 'sms'\nsms.send('+32xxxxxxxx','your text')"
		  },
		  {
			text: 'send a flash sms to multiple receivers',
			leaf: true,
			cls: 'file',
			script: "--assumes the daemon is running\nrequire 'sms'\nsms.send({'+32xxxxxxxx','+33xxxxxx'},'your text',true)"
		  },
		  {
			text: 'send a sms that requires to be acknowledged, to a cascading list of multiple receivers',
			leaf: true,
			cls: 'file',
			script: "--assumes the daemon is running\n--to activate the ack feature specify the time allowed to acknowledge in seconds as fourth argument\n--and by specifying a numbered list of phone numbers, with n1, n2, etc.\nrequire 'sms'\nsms.send({n1='+32xxxxxxxx',n2='+33xxxxxx'},'your text',false,120)"
		  },
		  {
			text: 'exclude a sender from the sms communication',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--this could be the service number of your provider, to assure that the service messages from the operator are not processed, only reported\nsms.blackList = {'+32477306003'} --you can add as many as you want to this table, adding '+32' will exclude all numbers from Belgium"
		  },
		  {
			text: 'include a sender in a blacklisted range',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--whiteList takes priority over blackList\nsms.whiteList = {'+32yourWhiteListedNumber'} --you can add as many as you want to this table, adding '+32477233' will allow all numbers starting with that string"
		  },
		  {
			text: 'define what action to take on an operator sms',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\nsms.operatorList = {'+32yourOperatorNumber'} --you can add as many as you want to this table\nsms.operatorAction(msg)\n    --your code here to define what to do with the operator 'msg'\nend"
		  },
		  {
			text: 'define alias for GA commands',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--if an alias is only defined in 'set', it will default also for 'get' \nsms.shortcuts = {\n\t['set']={\n\t\tverg = '0/3/9',\n\t\tkkn = {'0/1/7','0/1/9'},\n\t},\n\t['get']={\n\t\tkkn = {'0/0/7','0/0/9'},\n\t},\n}"
		  },
		]
	  },
	  {
		text: 'mail functions',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'mail client daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--for all receiving mail functionality it is required to run the daemon in a resident script with delay of 30-60s\n--the deamon assures that sms are send and received/processed by the default rules:\n--  - the part of the body text between <LM><\LM>-tags is treated as lua code\n--  - if no tags are present, the full body is treated aslua code\n--  - additionally, lua- and tx-files in attachement are treated as lua code\n\nmail = require 'mailplus'\n\nmail.processMails()"
		  },
		  {
			text: 'mail client config',
			leaf: true,
			cls: 'file',
			script: "--add to the startup script, (or less optimal, to the daemon, prior to the daemon start command).\nmail = require 'mailplus'\nlocal mailcfg={}\n--format of mailcfg with default values:\nmailcfg.smtp={\n\thost  = '', --your smtp host. If using FQN, then DNS must have been set\n\tssl       = '' ,  -- can be '', 'tslv1', 'sslv23', 'sslv3' -> gmail: 'sslv23'\n\tport      = 25,   -- gmail-SSL: 465\n\t--user      = '', --do not set user/password if your smtp does not expect it\n\t--password  = '',\n\tfrom      = ''    -- format VERY important, example: '\"KNXlogic.be LM2\" <knxlogic@mydomain.eu>',\n}\nmailcfg.pop = {\n\thost   = '', --your pop server\n\tport      = 110,  -- gmail SSL: 995\n\tuser      = '',\n\tpassword  = '',\n\tssl       = '' ,  -- can be '', 'tslv1', 'sslv23', 'sslv3' -> gmail: 'sslv23'\n\tnoop      = false ,  --when debug shows losing connection at initialisation, try set to true\n}\nmail.setMailConfig(mailcfg)\n"
		  },
		  {
			text: 'send an email',
			leaf: true,
			cls: 'file',
			script: "--assumes the mail configuration is set\nmail = require('mailplus')\nmail.send('to', 'cc', 'bcc', 'yourSubject','your text')  --to, cc and bcc can be nil(=not applicable), a address string or a table of addresses"
		  },
		  {
			text: 'reply to an email (in email)',
			leaf: true,
			cls: 'file',
			script: "--this only works as a command in an email: will reply the wanted string:\nmail.reply('yourtext')"
		  },
		  {
			text: 'exclude mailaddresses from the mail communication',
			leaf: true,
			cls: 'file',
			script: "--to insert in the daemon script\n--you can add as many as you want to this table, adding a substring like '@' will exclude all addresses with this part in the address (all in this case), except those whiteListed\nmail.setBlackList = {'a@b.ce','.be'} "
		  },
		  {
			text: 'allow mailaddresses to the mail communication',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--you can add as many as you want to this table, adding a substring will exclude all addresses with this part in the address\nmail setWhiteList = {'a@b.be','mydom.be'} "
		  },
		]
	  },
	  {
		text: 'Google/Exchange ICS Calendar daemon',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'gCal daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--run this as a resident script with 60s interval\n\nif not ready then\n    gCal = require('gCal')\n  \n    --specify the url where to get the ics file\n    gCal.url = 'https://www.google.com/calendar/ical/your@mail.com/private-yourPrivateKey/basic.ics'\n\n    --add here your customised action definition if wanted\n    --default actions are to write a value to the address specified in the title/name field\n    --value in description field is sent at event start\n    --value in location field is sent at event end\n\n    ready = true\nend\n\ngCal.run()\n\n"
		  },
		  {
			text: 'customising actions',
			leaf: true,
			cls: 'file',
			script: "    --add this to the daemon to overide the default actions, based on the event name, description and location\n\n    gCal.EventStart = function(MyEvent)\n        --put your code here: what to do when the event starts.\n        --The MyEvent table has the following info: start, end, name, description, location\n        --exemple:\n        --grp.write('12/1/1',1,dt.bool)\n        --grp.write('12/1/2',MyEvent['name'],dt.string)\n    end\n    gCal.EventEnd = function(MyEvent)\n        --put your code here: what to do when the event ends.\n        --The MyEvent table has the following info: start, end, name, description, location\n        --exemple:\n        --grp.write('12/1/1',0,dt.bool)\n        --grp.write('12/1/2',MyEvent['name'],dt.string)\n    end\n\n"
		  },
		  {
			text: 'customising actions: lua snippet',
			leaf: true,
			cls: 'file',
			script: "    --add this to the daemon to overide the default actions which are based on the event name, description and location\n    --the following code will execute the text in the description field at the start of the event as a lua code snippet\n\n	  gCal.EventStart = function(MyEvent)\n\n	    local f, emsg = loadstring(MyEvent['description']:gsub('\92\92','\92'):gsub('\\,',','):gsub('\\;',';'):gsub('\\n','\10'):gsub('\\N','\10'))\n    	if gCal.DEBUG>=2 then log('gCal','running code snippet',MyEvent['description']) end\n    	if f then\n      		f()\n    	else\n      		alert('gCal: invalid Code Snippet: '.. emsg)\n      	end\n    end\n\n"
		  },
		  {
			text: 'customise refresh time',
			leaf: true,
			cls: 'file',
			script: "    --add this to the daemon to overide the default calendar refresh time (in minutes), default is 20 minutes\n\n    gCal.reloadint = 20\n\n"
		  },
		  {
			text: 'action offset timing',
			leaf: true,
			cls: 'file',
			script: "    --add this to the daemon to specify the offset (in minutes) between the event start or end and the start or end actions\n    --a positieve value postpones the actions\n\n    gCal.startOffset = 0\n    gCal.endOffset = 0\n\n"
		  },
		]
	  },
	  {
		text: 'mPower mFi daemon',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'mPower daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--run this as a resident script with 0s interval\n\nif not ready then\n    mPower = require('mPower')\n    --configure your mFi mPower configuration: host (= IP address of the mFi) and groupaddresses to use\n    mPower.mPowerConfig= {\n        name 			= 'mFi-basement', -- the common name of your mPower for object naming\n        host 			= 'your mFi IP address',\n        password 		= 'ubnt', --is the default password. Only modify if you changed it.\n        --add the following list of communication objects for each socket, replacing 1 by 2, 3, ... 6 as needed\n        --in the tables, specify as many group addresses as wanted. Group addresses do not need to be unique\n        relay1_cmd 		= {'13/0/1'},\n        enabled1_cmd 	= {'13/0/2'},\n        reset1_cmd 		= {'13/0/3'},\n        relay1			= {'13/1/1'},\n        enabled1		= {'13/1/2'},\n        output1			= {'13/1/4'},\n        v_rms1			= {'13/1/5'},\n        i_rms1			= {'13/1/6'},\n        pf1				= {'13/1/7'},\n        active_pwr1		= {'13/1/8'},\n        energy_sum1		= {'13/1/9'},\n    }\n\n    ready = true\nend\n\nmPower.run()\n\n"
		  },
		  {
			text: 'change object datatype',
			leaf: true,
			cls: 'file',
			script: "\n    --add this (partially) to the daemon to override the default datatypes for the communication objects\n\n    mPower.datatype = {\n        reset = dt.bool,\n        relay = dt.bool,\n        enabled = dt.bool,\n        output = dt.bool,\n        v_rms  = dt.float16,\n        i_rms  = dt.float16,\n        pf     = dt.float16,\n        active_pwr = dt.float16,\n        energy_sum = dt.float16,\n    }\n "
		  },
		  {
			text: 'status update send threshold',
			leaf: true,
			cls: 'file',
			script: "\n    --add this to the daemon to override the default bus send threshold for sending status object updates (below are the deafult values or 0 when not specified)\n    --by setting a very large threshold you will stop automatically sending the new values. In such case the actual value will only be send when a read telegram is received\n\n    mPower.threshold = {\n        v_rms      = 1,\n        i_rms      = 0.01,\n        pf         = 0.01,\n        active_pwr = 1,\n        energy_sum = 1,\n    }\n"
		  },
		]
	  },
	  {
		text: 'ComfoLib - Zehnder-StorkAir',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'ComfoWay daemon (required) - local serial port',
			leaf: true,
			cls: 'file',
			script: "--This is the main ComfoWay daemon, using the configuration from the ComfoWay tab\n--Requires library V1.15 or higher\n--It must be run as resident script with 0 sec interval.\n\nreloadCodeOnly=storage.get('comfoway_reload',true)\n\n\nif not ready or reloadCodeOnly then\n\n  storage.set('KNXlogic_whr_lic','paste your licence key')\n\n	if not ready then os.sleep(10) end\n\n	comfoway = require('whr')\n\n	if not comfoway.GA then alert('ComfoWay daemon not started: license missing') os.sleep(60) return end\n\n	if reloadCodeOnly then \n		storage.set('comfoway_reload',false)\n	end\n	\n	comfoway.readConfigDB()\n  \n	comfoway.initConfig()\n	\n 	ready = true\n\nend\n\n--the actual daemon for a RS232 connected unit\nif ready then comfoway.run() end\n"
		  },
		  {
			text: 'Additional ComfoLib daemon (required) - local serial port',
			leaf: true,
			cls: 'file',
			script: "-- This is the ComfoLib daemon for an additional Unit, using the configuration from the specified table\n--Requires library V1.15 or higher\n-- first call: define mapping\nif not ready then\n\n    --if not ready then os.sleep(10) end\n\n    comfoway = require('whr')\n    if not comfoway.GA then alert('ComfoLib daemon not started: license missing') os.sleep(60) return end\n\n	-------------------------------------------------------\n	-- START of user configuration area\n	--\n  	-- give the unit an id to be able to differentiate object names and tags\n  	comfoway.comfolibID = 'whr350'\n  \n	-- you can use this main and sub variable to choose a specific range of objects. Alternatively, type them manually for all objects	\n	local main = 6\n	local sub=2\n\n	-- specification of the command group addresses\n	comfoway.GA.command={\n		volume = main..'/'..sub..'/0',\n    	volumeMan = main..'/'..sub..'/60',\n    	volumeAuto = main..'/'..sub..'/61',\n    	volume0 = main..'/'..sub..'/65',\n    	volume1 = main..'/'..sub..'/66',\n    	volume2 = main..'/'..sub..'/67',\n    	volume3 = main..'/'..sub..'/68',\n		Tcomfort = main..'/'..sub..'/1', \n		airSupply = main..'/'..sub..'/2',\n		airExhaust = main..'/'..sub..'/3',\n		filterReset = main..'/'..sub..'/5',\n		errorReset = main..'/'..sub..'/6',\n		filterTime = main..'/'..sub..'/7',\n        analogAutoMode = main..'/'..sub..'/70',\n        analog1set = main..'/'..sub..'/71',\n        analog2set = main..'/'..sub..'/72',\n        analog3set = main..'/'..sub..'/73',\n        analog4set = main..'/'..sub..'/74',\n		RS232      = main..'/'..sub..'/8', \n    	supplyFanVolA = main..'/'..sub..'/90',\n    	supplyFanVol1 = main..'/'..sub..'/91',\n    	supplyFanVol2 = main..'/'..sub..'/92',\n    	supplyFanVol3 = main..'/'..sub..'/93',\n    	exhaustFanVolA = main..'/'..sub..'/94',\n    	exhaustFanVol1 = main..'/'..sub..'/95',\n    	exhaustFanVol2 = main..'/'..sub..'/96',\n    	exhaustFanVol3 = main..'/'..sub..'/97',\n    	boostTime 		= main..'/'..sub..'/85',\n    	boostActive		= main..'/'..sub..'/86',\n	}\n\n	-- specification of the status group addresses\n	--   optional fiels:\n	--     int = update interval to override the default; nil = default\n	--     upd = only update the value internally, do not put it on the bus. true or false\n	--     wrold = write the old value first, before the update of the new value. true or false\n	comfoway.GA.status={\n		-- [[\n		volume   = main..'/'..sub+1 ..'/0',\n    	volumeMan = main..'/'..sub+1 ..'/60',\n    	volumeAuto = main..'/'..sub+1 ..'/61',\n    	volume0 = main..'/'..sub+1 ..'/65',\n    	volume1 = main..'/'..sub+1 ..'/66',\n    	volume2 = main..'/'..sub+1 ..'/67',\n    	volume3 = main..'/'..sub+1 ..'/68',\n		Tcomfort = {ga= main..'/'..sub+1 ..'/1', int=nil}, --]]\n		airSupply = main..'/'..sub+1 ..'/2',\n		airExhaust = main..'/'..sub+1 ..'/3',\n		filterTime = main..'/'..sub+1 ..'/7',\n		RS232 = main..'/'..sub+1 ..'/8',\n		T1 = {ga=main..'/'..sub+1 ..'/11', int=nil},\n		T2 = {ga=main..'/'..sub+1 ..'/12', int=nil},\n		T3 = {ga=main..'/'..sub+1 ..'/13', int=nil},\n		T4 = {ga=main..'/'..sub+1 ..'/14', int=nil},\n		Tge     = {ga=main..'/'..sub+1 ..'/21', int=nil},\n		Tah		= {ga=main..'/'..sub+1 ..'/22', int=nil},\n		Tkh		= {ga=main..'/'..sub+1 ..'/23', int=nil},\n		Tenth	= {ga=main..'/'..sub+1 ..'/24', int=nil},\n		Menth	= {ga=main..'/'..sub+1 ..'/25', int=nil},\n		supplyFan  = {ga=main..'/'..sub+1 ..'/26', int=nil},\n		exhaustFan = {ga=main..'/'..sub+1 ..'/27', int=nil},\n		bypassValve	= {ga=main..'/'..sub+1 ..'/28', int=nil},\n		preheatValve= {ga=main..'/'..sub+1 ..'/29', int=nil},\n		errorActual = {ga=main..'/'..sub+1 ..'/30', int=nil},\n		errorLast   = {ga=main..'/'..sub+1 ..'/31', int=nil},\n		filterDirty = {ga=main..'/'..sub+1 ..'/32', int=nil},\n		errorActualStr = {ga=main..'/'..sub+1 ..'/33', int=nil},\n		errorLastStr   = {ga=main..'/'..sub+1 ..'/34', int=nil},\n		timerA       = main..'/'..sub+1 ..'/40',\n		timer1       = main..'/'..sub+1 ..'/41',\n		timer2       = main..'/'..sub+1 ..'/42',\n		timer3       = main..'/'..sub+1 ..'/43',\n		timerFrost   = main..'/'..sub+1 ..'/44',\n		timerPreheat = main..'/'..sub+1 ..'/45',\n		timerBypass  = main..'/'..sub+1 ..'/46',\n		timerFilter  = main..'/'..sub+1 ..'/47',\n		timerFilterWk= main..'/'..sub+1 ..'/48',\n		errorComm    = main..'/'..sub+1 ..'/50', \n        analogAutoMode = main..'/'..sub+1 ..'/70',\n        analog1set = main..'/'..sub+1 ..'/71',\n        analog2set = main..'/'..sub+1 ..'/72',\n        analog3set = main..'/'..sub+1 ..'/73',\n        analog4set = main..'/'..sub+1 ..'/74',\n        analog1value = main..'/'..sub+1 ..'/75',\n        analog2value = main..'/'..sub+1 ..'/76',\n        analog3value = main..'/'..sub+1 ..'/77',\n        analog4value = main..'/'..sub+1 ..'/78',\n		bypassActive= main..'/'..sub+1 ..'/80',\n		EWTActive	= main..'/'..sub+1 ..'/81',\n		frostActive	= main..'/'..sub+1 ..'/82',\n		afHeatActive= main..'/'..sub+1 ..'/83',\n		hoodActive	= main..'/'..sub+1 ..'/84',\n    	supplyFanVolA = main..'/'..sub+1 ..'/90',\n    	supplyFanVol1 = main..'/'..sub+1 ..'/91',\n    	supplyFanVol2 = main..'/'..sub+1 ..'/92',\n    	supplyFanVol3 = main..'/'..sub+1 ..'/93',\n    	exhaustFanVolA = main..'/'..sub+1 ..'/94',\n    	exhaustFanVol1 = main..'/'..sub+1 ..'/95',\n    	exhaustFanVol2 = main..'/'..sub+1 ..'/96',\n    	exhaustFanVol3 = main..'/'..sub+1 ..'/97',\n    	boostTime 		= main..'/'..sub+1 ..'/85',\n    	boostActive		= main..'/'..sub+1 ..'/86',\n	}\n\n  --optionally, you can use a rs232-IP convertor and connect as tcp client or use a different serial port than the default RS232, \n  --then add the following general configuration statement: uncomment and addapt the line below\n  --comfoway.genConf.comfoair.connect={type = 'tcpclient', host = '192.168.255.11', port = 1470} \n\n	--\n	-- END of user configuration area\n	-- do not modify below\n	--\n	-------------------------------------------------------\n\n\n	comfoway.readConfigDB()\n  \n	comfoway.initConfig()\n	\n	ready=true\nend\n\n--the actual daemon for a RS232 connected unit\nif ready then comfoway.run() end\n"
		  },
		  {
			text: 'Available WHR functions',
			leaf: true,
			cls: 'file',
			script: "--If you want to know all implemented status and commands use the following prior to your own config and look in the log\nrequire'whr'\nlog(whr.GA)"
		  },
		  {
			text: 'Adv.config: custom status update interval',
			leaf: true,
			cls: 'file',
			script: "\t--override the default update interval use this in your config (interval in seconds): \n\tvolume = {ga='6/0/0', int=30},"
		  },
		  {
			text: 'Adv.config: multiple group addresses',
			leaf: true,
			cls: 'file',
			script: "\t--write a status to several group addresses or read a command from multiple GA's: \n\tvolume = {'6/0/0', '1/0/0', '7/8/9'},"
		  },
		  {
			text: 'Adv.config: status with custom update interval and multiple group addresses',
			leaf: true,
			cls: 'file',
			script: "\t--write status to multiple GA at custom interval\n\tvolume = {ga={'6/0/0', '1/0/0', '7/8/9'}, int=30},"
		  },
		]
	  },
	  {
		text: 'SPClib - SPC intrusion panels',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'SPCway daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--This is the main SPCway daemon, using the configuration from the SPCway tab\n--It must be run as resident script with 0 sec interval\n\nreloadCodeOnly=storage.get('spcway_reload',true)\n\nif not ready or reloadCodeOnly then\n    -- initialise at startup\n\nspcway = require('spcway')\n    if not spcway.commands then alert('SPCway daemon not started: license missing') os.sleep(60) return end\n\n    if reloadCodeOnly then \n		storage.set('spcway_reload',false)\n	end\n	\n	spcway.readConfigDB()\n  \n    if spcway.genConfig.panelID=='' or spcway.genConfig.myID=='' or spcway.genConfig.TCPport=='' then\n      if not genReported then alert('EDP TCP daemon: base connection parameter(s) missing -> waiting 5s') end\n      --genReported = true\n      os.execute('sleep ' .. tonumber(5))\n    else\n\n    spcway.initConfig()\n	\n    if not reloadCodeOnly or not ready then\n	  	spcway.initEDPTCPlistener()\n\n	    --start KNX telegram handlers after creating GA's\n  	  spcway.initKNXhandlers()\n	  end\n	\n  	ready = true\n  end\nend\n\nif ready then spcway.run() end"
		  },
		  {
			text: 'Additional SPClib daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--This is the SPClib daemon to connect an additional SPC panel, using the configuration from the specified table\n--It must be run as resident script with 0 sec interval\n\nif not ready then\n    -- initialise at startup\n\n    spcway = require('spcway')\n    if not spcway.commands then alert('SPCway daemon not started: license missing') os.sleep(60) return end\n\n	spcway.genConfig={}\n	spcway.SIAevents={}\n	spcway.commands={}\n	spcway.status={}\n	spcway.readConfigDB()\n  \n    if spcway.genConfig.panelID=='' or spcway.genConfig.myID=='' or spcway.genConfig.TCPport=='' then\n      if not genReported then alert('EDP TCP daemon: base connection parameter(s) missing -> waiting 5s') end\n      --genReported = true\n      os.execute('sleep ' .. tonumber(5))\n    else\n\n    spcway.initConfig()\n	\n    if not reloadCodeOnly or not ready then\n	  	spcway.initEDPTCPlistener()\n\n	    --start KNX telegram handlers after creating GA's\n  	  spcway.initKNXhandlers()\n	  end\n	\n  	ready = true\n  end\nend\n\nif ready then spcway.run() end"
		  },
		]
	  },
	  {
		text: 'Siemens SPC - X10 receiver',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'SPC X10 daemon (required) - local serial port',
			leaf: true,
			cls: 'file',
			script: "--for all X10 receiver functionality it is required to run the deamon in a resident script for delay 0\n\nrequire 'spcx10'\n\n--configure basic groupaddress allocation/linking\nspcx10.mapping = {\n\tA1 = '3/0/0',\n\tA2 = '3/0/1',\n}\n\nspcx10.run('/dev/ttyS5') --4 available serial ports (232 and 485) on your device are ttyS2, ttyS3, ttyS4, ttyS5, from left to right  "
		  },
		  {
			text: 'Adv. config: link with simple activation switch',
			leaf: true,
			cls: 'file',
			script: "\t--need this GA link only at certain moments (eg. only at night):\n\tA3 = {ga='3/0/2', act = grp.getvalue('night')}"
		  },
		  {
			text: 'Adv. config: link with function activation switch',
			leaf: true,
			cls: 'file',
			script: "--a function as activation switch (when too complex to call every second):\nif running then\n\tisA3act = function()\n\t\t--your code\n\t\treturn yourvalue\n\tend\n\n\tnspcx10.mapping = {\n\t\tA1 = '3/0/0',\n\t\tA2 = '3/0/1',\n\t\tA3 = {ga='3/0/2', act = isA3act},\n\t} \n\trunning=true\nend"
		  },
		]
	  },
    ]
  },
]
